#-*- coding: utf-8 -*-
from __future__ import unicode_literals

from bs4 import BeautifulSoup, NavigableString

from django.forms.models import modelform_factory
from django.test import TestCase as DjangoTestCase
from django.utils.html import conditional_escape

from wysihtml5.tests.models import ModelTest
from wysihtml5.widgets import (Wysihtml5TextareaWidget,
                               render_cmd_icon, render_cmd_dialog)


class Wysihtml5ToolbarTestCase(DjangoTestCase):
    def setUp(self):
        ModelForm = modelform_factory(ModelTest)
        # self.soup = BeautifulSoup(unicode(ModelForm()))
        self.soup = BeautifulSoup(ModelForm().__str__())

    def test_command_disabled_in_settings(self):        
        # check an active command and the disabled one
        cmd_format_block = self.soup.find('span', attrs={
                'data-wysihtml5-command-group': 'formatBlock'})
        cmd_fore_color = self.soup.find('span', attrs={
                'data-wysihtml5-command-group': 'foreColor'})
        self.assert_(cmd_format_block != None)
        self.assert_(cmd_fore_color == None) # tests.settings disabled

    def test_dialog_disabled_in_settings(self):        
        # check the active dialog and the disabled one
        dialog_insert_image = self.soup.find('div', attrs={
                'data-wysihtml5-dialog': 'insertImage'})
        dialog_create_link = self.soup.find('span', attrs={
                'data-wysihtml5-dialog': 'createLink'})
        self.assert_(dialog_insert_image != None)
        self.assert_(dialog_create_link == None) # tests.settings disabled


class Wysihtml5TextareaWidgetTestCase(DjangoTestCase):
    def test_render_wysihtml5admintextarea_widget(self):
        neilmsg = ModelTest.objects.create(
            first_text="One small step for man", 
            second_text="One giant leap for mankind")
        w = Wysihtml5TextareaWidget()
        rendered = conditional_escape(w.render("test", neilmsg.second_text))
        expected = '\
<div style="display:inline-block"><div id="unknown-toolbar" class="wysihtml5-editor-toolbar">\
  <div class="commands">\
    <span data-wysihtml5-command-group="formatBlock" title="Format text header" class="heading-selector">\
      <div>\
        <span data-wysihtml5-command="formatBlock" data-wysihtml5-command-value="h1">H1</span>\
        <span data-wysihtml5-command="formatBlock" data-wysihtml5-command-value="h2">H2</span>\
        <span data-wysihtml5-command="formatBlock" data-wysihtml5-command-value="h3">H3</span>\
        <span data-wysihtml5-command="formatBlock" data-wysihtml5-command-value="h4">H4</span>\
        <span data-wysihtml5-command="formatBlock" data-wysihtml5-command-value="h5">H5</span>\
        <span data-wysihtml5-command="formatBlock" data-wysihtml5-command-value="h6">H6</span>\
      </div>\
    </span>\
    <span data-wysihtml5-command="formatBlock" title="Make a paragraph block" data-wysihtml5-command-value="p" class="command format-block-p"></span>\
    <span data-wysihtml5-command="bold" title="Make text bold (CTRL + B)" class="command"></span>\
    <span data-wysihtml5-command="italic" title="Make text italic (CTRL + I)" class="command"></span>\
    <span data-wysihtml5-command="underline" title="Underline text (CTRL + U)" class="command"></span>\
    <span data-wysihtml5-command="justifyLeft" title="Paragraph left justified" class="command"></span>\
    <span data-wysihtml5-command="justifyCenter" title="Paragraph center justified" class="command"></span>\
    <span data-wysihtml5-command="justifyRight" title="Paragraph right justified" class="command"></span>\
    <span data-wysihtml5-command="insertOrderedList" title="Insert an ordered list" class="command"></span>\
    <span data-wysihtml5-command="insertUnorderedList" title="Insert an unordered list" class="command"></span>\
    <span data-wysihtml5-command="insertImage" title="Insert an image" class="command insert-image"></span>\
    <span data-wysihtml5-command="insertHTML" title="Insert a quote" class="command" data-wysihtml5-command-value="<blockquote>quote</blockquote>"></span>\
    <span data-wysihtml5-action="change_view" title="Show HTML" class="action" unselectable="on"></span>\
  </div>\
  <div class="wysihtml5-dialogs">\
    <div data-wysihtml5-dialog="insertImage" style="display:none">  <label>Image:</label>&nbsp;  <input data-wysihtml5-dialog-field="src" value="http://">  <a data-wysihtml5-dialog-action="save" class="button">Ok</a>&nbsp;  <a data-wysihtml5-dialog-action="cancel" class="button">Cancel</a></div>  </div></div><textarea rows="25" cols="40" name="test" class="vLargeTextField">One giant leap for mankind</textarea></div>\
<script>\
new wysihtml5.Editor("unknown",{ name: null, style: true, toolbar: "unknown-toolbar", autoLink: true, parserRules: wysihtml5ParserRules, parser: wysihtml5.dom.parse || Prototype.K, composerClassName: "wysihtml5-editor", bodyClassName: "wysihtml5-supported", useLineBreaks: true, stylesheets: ["/static/wysihtml5/css/stylesheet.css"], placeholderText: null, allowObjectResizing: true, supportTouchDevices: true });\
</script>'
        self.maxDiff = None
        self.assertHTMLEqual(expected, rendered)



#-*- coding: utf-8 -*-

from django.db import models

from wysihtml5.fields import Wysihtml5TextField

class ModelTest(models.Model):
    first_text = models.TextField()
    second_text = Wysihtml5TextField()



#-*- coding: utf-8 -*-

from django.conf import settings

WYSIHTML5_EDITOR = {
    # Give the editor a name, the name will also be set as class 
    # name on the iframe and on the iframe's body
    'name': 'null',
    # Whether the editor should look like the textarea (by adopting styles)
    'style': 'true',
    # Id of the toolbar element, pass false if you don't want 
    # any toolbar logic
    'toolbar': 'null',
    # Whether urls, entered by the user should automatically become 
    # clickable-links
    'autoLink': 'true',
    # Object which includes parser rules (set this to 
    # examples/rules/spec.json or your own spec, otherwise only span 
    # tags are allowed!)
    'parserRules': 'wysihtml5ParserRules',
    # Parser method to use when the user inserts content via copy & paste
    'parser': 'wysihtml5.dom.parse || Prototype.K',
    # Class name which should be set on the contentEditable element in 
    # the created sandbox iframe, can be styled via the 'stylesheets' option
    'composerClassName': '"wysihtml5-editor"',
    # Class name to add to the body when the wysihtml5 editor is supported
    'bodyClassName': '"wysihtml5-supported"',
    # By default wysihtml5 will insert <br> for line breaks, set this to
    # false to use <p>
    'useLineBreaks': 'true',
    # Array (or single string) of stylesheet urls to be loaded in the 
    # editor's iframe
    'stylesheets': '["%s"]' % (settings.STATIC_URL + 
                               "wysihtml5/css/stylesheet.css"),
    # Placeholder text to use, defaults to the placeholder attribute 
    # on the textarea element
    'placeholderText': 'null',
    # Whether the composer should allow the user to manually resize 
    # images, tables etc.
    'allowObjectResizing': 'true',
    # Whether the rich text editor should be rendered on touch devices 
    # (wysihtml5 >= 0.3.0 comes with basic support for iOS 5)
    'supportTouchDevices': 'true'
}

WYSIHTML5_TOOLBAR = {
    "formatBlockHeader": { 
        "active": True,
        "command_name": "formatBlock",
        "render_icon": "wysihtml5.widgets.render_formatBlockHeader_icon"
    },
    "formatBlockParagraph": { 
        "active": True,
        "command_name": "formatBlock",
        "render_icon": "wysihtml5.widgets.render_formatBlockParagraph_icon"
    },
    "bold": { 
        "active": True,
        "command_name": "bold",
        "render_icon": "wysihtml5.widgets.render_bold_icon"
    },
    "italic": { 
        "active": True,
        "command_name": "italic",
        "render_icon": "wysihtml5.widgets.render_italic_icon"
    },
    "underline": { 
        "active": True,
        "command_name": "underline",
        "render_icon": "wysihtml5.widgets.render_underline_icon"
    },
    "justifyLeft": { 
        "active": True,
        "command_name": "justifyLeft",
        "render_icon": "wysihtml5.widgets.render_justifyLeft_icon"
    },
    "justifyCenter": { 
        "active": True,
        "command_name": "justifyCenter",
        "render_icon": "wysihtml5.widgets.render_justifyCenter_icon"
    },
    "justifyRight": { 
        "active": True,
        "command_name": "justifyRight",
        "render_icon": "wysihtml5.widgets.render_justifyRight_icon"
    },
    "insertOrderedList": { 
        "active": True,
        "command_name": "insertOrderedList",
        "render_icon": "wysihtml5.widgets.render_insertOrderedList_icon"
    },
    "insertUnorderedList": { 
        "active": True,
        "command_name": "insertUnorderedList",
        "render_icon": "wysihtml5.widgets.render_insertUnorderedList_icon"
    },
    "insertImage": { 
        "active": True,
        "command_name": "insertImage",
        "render_icon": "wysihtml5.widgets.render_insertImage_icon",
        "render_dialog": "wysihtml5.widgets.render_insertImage_dialog"
    },
    "createLink": { 
        "active": True,
        "command_name": "createLink",
        "render_icon": "wysihtml5.widgets.render_createLink_icon",
        "render_dialog": "wysihtml5.widgets.render_createLink_dialog"
    },
    "insertHTML": { 
        "active": True,
        "command_name": "insertHTML",
        "command_value": "<blockquote>quote</blockquote>",
        "render_icon": "wysihtml5.widgets.render_insertHTML_icon"
    },
    "foreColor": { 
        "active": True,
        "command_name": "foreColor",
        "render_icon": "wysihtml5.widgets.render_foreColor_icon"
    },
    "changeView": { 
        "active": True,
        "command_name": "change_view",
        "render_icon": "wysihtml5.widgets.render_changeView_icon"
    },
}

# This is necessary to protect the field of content in cases where
# the user disables JavaScript in the browser, so that Wysihtml5 can't
# do the filter job.
WYSIHTML5_ALLOWED_TAGS = ('h1 h2 h3 h4 h5 h6 div p b i u'
                          ' ul ol li span img a blockquote')



#-*- coding: utf-8 -*-

from wysihtml5.fields import Wysihtml5TextField
from wysihtml5.widgets import Wysihtml5TextareaWidget

class AdminWysihtml5TextFieldMixin(object):
    """Mixin for ModelAdmin subclasses to provide custom widget for ``Wysihtml5TextField`` fields."""
    def formfield_for_dbfield(self, db_field, **kwargs):
        if isinstance(db_field, Wysihtml5TextField):
            return db_field.formfield(widget=Wysihtml5TextareaWidget)
        sup = super(AdminWysihtml5TextFieldMixin, self)
        return sup.formfield_for_dbfield(db_field, **kwargs)



#-*- coding: utf-8 -*-
from __future__ import unicode_literals

from bs4 import BeautifulSoup, NavigableString

from django import forms
from django.forms.models import modelform_factory
from django.forms.widgets import Textarea
from django.test import TestCase as DjangoTestCase

from wysihtml5.tests.models import ModelTest
from wysihtml5.widgets import Wysihtml5TextareaWidget


class Wysihtml5TextFieldTestCase(DjangoTestCase):
    def test_widget_for_wysihtml5textfield_model_field(self):
        form = modelform_factory(ModelTest)()
        first_widget = form.fields.get("first_text").widget
        second_widget = form.fields.get("second_text").widget
        self.assertEqual(first_widget.__class__, Textarea)
        self.assertEqual(second_widget.__class__, Wysihtml5TextareaWidget)


class KeepTagsTestCase(DjangoTestCase):
    # Test this in the browser and example site by disabling JavaScript
    def setUp(self):
        ModelTest.objects.create(
            first_text="Something not important here", 
            second_text=(
                # Allowed Tags:
                'A text with all the tags is coming here'
                '<h1>Header 1</h1>'
                '<h2>Header 2</h2>'
                '<h3>Header 3</h3>'
                '<h4>Header 4</h4>'
                '<h5>Header 5</h5>'
                '<h6>Header 6</h6>'
                '<h7>Header 7</h7>'
                '<div>A DIV Element</div>'
                '<p>A Paragraph</p>'
                '<b>A B Element</b>'
                '<i>An I Element</i>'
                '<u>An U Element</u>'
                '<ul>An UL Element<li>And a LI Element</li></ul>'
                '<ol>An OL Element<li>And a LI Element</li></ol>'
                '<span>A SPAN element</span>'
                '<img src="http://blahblahbla.png">'
                '<a href="#">An Anchor Element</a>'
                '<blockquote>Quoting a quote!</blockquote>'

                # Disallowed Tags
                '<script>alert("this should not stay!")</script>'
                '<h7>A H7 header?</h7>'))
        self.obj = ModelTest.objects.get(pk=1)
        
    def test_keeptags_keep_the_allowed_tags(self):
        soup = BeautifulSoup(self.obj.second_text)
        self.assert_(soup.find("h1") != None)
        self.assert_(soup.find("h2") != None)
        self.assert_(soup.find("h3") != None)
        self.assert_(soup.find("h4") != None)
        self.assert_(soup.find("h5") != None)
        self.assert_(soup.find("h6") != None)
        self.assert_(soup.find("div") != None)
        self.assert_(soup.find("p") != None)
        self.assert_(soup.find("b") != None)
        self.assert_(soup.find("i") != None)
        self.assert_(soup.find("u") != None)
        self.assert_(soup.find("ul") != None)
        self.assert_(soup.find("ol") != None)
        self.assert_(soup.find("span") != None)
        self.assert_(soup.find("img") != None)
        self.assert_(soup.find("a") != None)
        self.assert_(soup.find("blockquote") != None)

    def test_keeptags_removes_not_specified_tags(self):
        soup = BeautifulSoup(self.obj.second_text)
        self.assert_(soup.find("script") == None)
        self.assert_(soup.find("h7") == None)



#-*- coding: utf-8 -*-

from django.db.models import fields

from wysihtml5.conf import settings
from wysihtml5.utils import keeptags
from wysihtml5.widgets import Wysihtml5TextareaWidget


class Wysihtml5TextField(fields.TextField):
    def __init__(self, *args, **kwargs):
        self.keep_tags = kwargs.pop('keep_tags', 
                                    settings.WYSIHTML5_ALLOWED_TAGS)
        super(Wysihtml5TextField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {"widget": Wysihtml5TextareaWidget}
        defaults.update(kwargs)
        return super(Wysihtml5TextField, self).formfield(**defaults)

    def pre_save(self, model_instance, add):
        value = super(Wysihtml5TextField, self).pre_save(model_instance, add)
        return keeptags(value, self.keep_tags)


try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^wysihtml5\.fields\.Wysihtml5TextField"])
except ImportError:
    pass



#-*- coding: utf-8 -*-

import os
import tempfile

from django.core.files.storage import FileSystemStorage

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE':   'django.db.backends.sqlite3', 
        'NAME':     'django_wysihtml5_test',
        'USER':     '', 
        'PASSWORD': '', 
        'HOST':     '', 
        'PORT':     '',
    }
}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'Europe/Madrid'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = ''

STATIC_URL = '/static/'

SECRET_KEY = 'v2824l&2-n+4zznbsk9c-ap5i)b3e8b+%*a=dxqlahm^%)68jn'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    # 'django.middleware.csrf.CsrfViewMiddleware',
    # 'django.contrib.auth.middleware.AuthenticationMiddleware',
)

ROOT_URLCONF = 'wysihtml5.tests.urls'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    os.path.join(os.path.dirname(__file__), "..", "templates"),
    os.path.join(os.path.dirname(__file__), "templates"),
)

INSTALLED_APPS = [
    "django.contrib.contenttypes",
    'django.contrib.sites',
    'wysihtml5',
    'wysihtml5.tests',
]

WYSIHTML5_TOOLBAR = {
    "foreColor": { "active": False },
    "createLink": { "active": False }
}



import six

from django.conf import settings as django_settings
from django.utils.functional import LazyObject

from wysihtml5.conf import defaults as app_settings


class LazySettings(LazyObject):
    def _setup(self):
        self._wrapped = Settings(app_settings, django_settings)

def update_dict_in_depth(a, b):
    """Updates dict a in depth with values of dict b (not for sequences)"""
    for k, v in six.iteritems(b):
        if a.get(k, None) and type(v) == dict:
                update_dict_in_depth(a[k], v)
        else:
            a[k] = v

class Settings(object):
    def __init__(self, *args):
        for item in args:
            for attr in dir(item):
                if attr == attr.upper():
                    setattr(self, attr, getattr(item, attr))

    def __setattr__(self, name, value):
        obj_attr = getattr(self, name, None)
        if obj_attr and type(obj_attr) == dict:
            update_dict_in_depth(obj_attr, value)
        else:
            object.__setattr__(self, name, value)

settings = LazySettings()



#-*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from django.core.exceptions import ImproperlyConfigured
from django.utils.functional import allow_lazy
from django.utils.importlib import import_module


def get_function(function_path):
    """
    import and return function from ``path.to.module.function`` argument
    """
    try:
        mod_name, func_name = function_path.rsplit('.', 1)
        mod = import_module(mod_name)
    except ImportError as e:
        raise ImproperlyConfigured(('Error importing module %s: "%s"' %
                                   (mod_name, e)))
    return getattr(mod, func_name)


def keeptags(value, tags):
    tags = [re.escape(tag) for tag in tags.split()]

    def _replacer(match):
        if match.group(1) in tags:
            return match.group(0)
        else:
            return ''

    return re.sub(r'</?([^> ]+).*?>', _replacer, value)
keeptags = allow_lazy(keeptags)



from setuptools import setup, find_packages
from setuptools.command.test import test

def run_tests(*args):
    from wysihtml5.tests import run_tests
    run_tests()

test.run_tests = run_tests

setup(
    name = "django-wysihtml5",
    version = "1.2b3",
    packages = find_packages(),
    include_package_data = True,
    license = "MIT",
    description = "Simple Django app that provides a Wysihtml5 rich text editor textarea widget.",
    long_description = "Simple Django app that provides a Wysihtml5 rich text editor textarea widget, with a complete command toolbar to give HTML format to your documents.",
    author = "Daniel Rus Morales",
    author_email = "inbox@danir.us",
    maintainer = "Daniel Rus Morales",
    maintainer_email = "inbox@danir.us",
    url = "http://pypi.python.org/pypi/django-wysihtml5/",
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Framework :: Django',
    ],
    test_suite = "dummy",
)



#-*- coding: utf-8 -*-

import os
import sys
import unittest

def setup_django_settings():
    os.chdir(os.path.join(os.path.dirname(__file__), ".."))
    sys.path.insert(0, os.getcwd())
    os.environ["DJANGO_SETTINGS_MODULE"] = "tests.settings"


def run_tests():
    if not os.environ.get("DJANGO_SETTINGS_MODULE", False):
        setup_django_settings()

    from django.conf import settings
    from django.test.utils import get_runner

    runner = get_runner(settings, "django.test.simple.DjangoTestSuiteRunner")
    test_suite = runner(verbosity=2, interactive=True, failfast=False)
    test_suite.run_tests(["wysihtml5"])


def suite():
    if not os.environ.get("DJANGO_SETTINGS_MODULE", False):
        setup_django_settings()
    else:
        from django.db.models.loading import load_app
        from django.conf import settings
        settings.INSTALLED_APPS = list(settings.INSTALLED_APPS) + \
                                  ['wysihtml5.tests']
        map(load_app, settings.INSTALLED_APPS)

    from wysihtml5.tests import fields, widgets

    testsuite = unittest.TestSuite([
        unittest.TestLoader().loadTestsFromModule(fields),
        unittest.TestLoader().loadTestsFromModule(widgets),
    ])
    return testsuite


if __name__ == "__main__":
    run_tests()



#-*- coding: utf-8 -*-
from __future__ import unicode_literals

import six

from django.contrib.admin.widgets import AdminTextareaWidget
from django.forms.util import flatatt
try:
    from django.utils.encoding import force_text
except ImportError:
    # Django < 1.4.2
    from django.utils.encoding import force_unicode as force_text
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _

from wysihtml5.conf import settings
from wysihtml5.utils import get_function


render_cmd_icon = {}
render_cmd_dialog = {}

def render_blank(id):
    return ''

def render_createLink_dialog(id):
    return '\
<div data-wysihtml5-dialog="createLink" style="display:none">\
  <label>%(_link_)s:</label>&nbsp;\
  <input data-wysihtml5-dialog-field="href" value="http://">\
  <a data-wysihtml5-dialog-action="save" class="button">%(_ok_)s</a>&nbsp;\
  <a data-wysihtml5-dialog-action="cancel" class="button">%(_cancel_)s</a>\
</div>' % { "_link_": _("Link"),
            "_ok_": _("Ok"),
            "_cancel_": _("Cancel")
        }


def render_insertImage_dialog(id):
    return '\
<div data-wysihtml5-dialog="insertImage" style="display:none">\
  <label>%(_link_)s:</label>&nbsp;\
  <input data-wysihtml5-dialog-field="src" value="http://">\
  <a data-wysihtml5-dialog-action="save" class="button">%(_ok_)s</a>&nbsp;\
  <a data-wysihtml5-dialog-action="cancel" class="button">%(_cancel_)s</a>\
</div>' % { "_link_": _("Image"),
            "_ok_": _("Ok"),
            "_cancel_": _("Cancel")
        }


def render_formatBlockHeader_icon(id):
    return '\
    <span data-wysihtml5-command-group="%(command_name)s" title="Format text header" class="heading-selector">\
      <div>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="h1">H1</span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="h2">H2</span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="h3">H3</span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="h4">H4</span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="h5">H5</span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="h6">H6</span>\
      </div>\
    </span>' % { "command_name": settings.WYSIHTML5_TOOLBAR['formatBlockHeader']['command_name'] }

def render_formatBlockParagraph_icon(id):
    return '<span data-wysihtml5-command="%(command_name)s" title="Make a paragraph block" data-wysihtml5-command-value="p" class="command format-block-p"></span>' % { "command_name": settings.WYSIHTML5_TOOLBAR['formatBlockParagraph']['command_name'] }

def render_bold_icon(id):
    return '<span data-wysihtml5-command="%(command_name)s" title="Make text bold (CTRL + B)" class="command"></span>' % { "command_name": settings.WYSIHTML5_TOOLBAR['bold']['command_name'] }

def render_italic_icon(id):
    return '<span data-wysihtml5-command="%(command_name)s" title="Make text italic (CTRL + I)" class="command"></span>' % { "command_name": settings.WYSIHTML5_TOOLBAR['italic']['command_name'] }

def render_underline_icon(id):
    return '<span data-wysihtml5-command="%(command_name)s" title="Underline text (CTRL + U)" class="command"></span>' % { "command_name": settings.WYSIHTML5_TOOLBAR['underline']['command_name'] }

def render_justifyLeft_icon(id):
    return '<span data-wysihtml5-command="%(command_name)s" title="Paragraph left justified" class="command"></span>' % { "command_name": settings.WYSIHTML5_TOOLBAR['justifyLeft']['command_name'] }

def render_justifyCenter_icon(id):
    return '<span data-wysihtml5-command="%(command_name)s" title="Paragraph center justified" class="command"></span>' % { "command_name": settings.WYSIHTML5_TOOLBAR['justifyCenter']['command_name'] }

def render_justifyRight_icon(id):
    return '<span data-wysihtml5-command="%(command_name)s" title="Paragraph right justified" class="command"></span>' % { "command_name": settings.WYSIHTML5_TOOLBAR['justifyRight']['command_name'] }

def render_insertOrderedList_icon(id):
    return '<span data-wysihtml5-command="%(command_name)s" title="Insert an ordered list" class="command"></span>' % { "command_name": settings.WYSIHTML5_TOOLBAR['insertOrderedList']['command_name'] }

def render_insertUnorderedList_icon(id):
    return '<span data-wysihtml5-command="%(command_name)s" title="Insert an unordered list" class="command"></span>' % { "command_name": settings.WYSIHTML5_TOOLBAR['insertUnorderedList']['command_name'] }

def render_insertImage_icon(id):
    return '<span data-wysihtml5-command="%(command_name)s" title="Insert an image" class="command insert-image"></span>' % { 'command_name': settings.WYSIHTML5_TOOLBAR['insertImage']['command_name'] }

def render_createLink_icon(id):
    return '<span data-wysihtml5-command="%(command_name)s" title="Insert a link" class="command create-link"></span>' % { 'command_name': settings.WYSIHTML5_TOOLBAR['createLink']['command_name'] }

def render_insertHTML_icon(id):
    return '<span data-wysihtml5-command="%(command_name)s" title="Insert a quote" class="command" data-wysihtml5-command-value="%(command_value)s"></span>'  % {  'command_name': settings.WYSIHTML5_TOOLBAR['insertHTML']['command_name'] , 'command_value': settings.WYSIHTML5_TOOLBAR['insertHTML']['command_value'] }

def render_foreColor_icon(id):
    return '\
      <span data-wysihtml5-command-group="%(command_name)s" title="Color the selected text" class="fore-color">\
      <div>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="silver" unselectable="on"></span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="gray" unselectable="on"></span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="maroon" unselectable="on"></span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="red" unselectable="on"></span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="purple" unselectable="on"></span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="green" unselectable="on"></span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="olive" unselectable="on"></span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="navy" unselectable="on"></span>\
        <span data-wysihtml5-command="%(command_name)s" data-wysihtml5-command-value="blue" unselectable="on"></span>\
      </div>\
    </span>' % { 'command_name': settings.WYSIHTML5_TOOLBAR['foreColor']['command_name'] }

def render_changeView_icon(id):
    return '<span data-wysihtml5-action="%(command_name)s" title="Show HTML" class="action" unselectable="on"></span>' % { 'command_name': settings.WYSIHTML5_TOOLBAR['changeView']['command_name'] }


class Wysihtml5TextareaWidget(AdminTextareaWidget):

    class Media:
        css = {
            'all': ('wysihtml5/css/toolbar.css',)
        }
        js = ('wysihtml5/js/advanced.js',
              'wysihtml5/js/wysihtml5-0.4.0pre.min.js')

    def __init__(self, attrs=None, editor_settings=None, toolbar_settings=None):
        if not attrs:
            attrs = {"rows": 25}
        elif not attrs.get("rows", False):
            attrs.update({"rows": 25})

        if editor_settings:
            self.editor_settings = editor_settings
        else: 
            self.editor_settings = settings.WYSIHTML5_EDITOR

        if toolbar_settings:
            self.toolbar_settings = toolbar_settings
        else:
            self.toolbar_settings = settings.WYSIHTML5_TOOLBAR

        self.render_cmd_icon = {}
        self.render_cmd_dialog = {}
        for k, v in six.iteritems(self.toolbar_settings):
            if v.get("active", False):
                self.render_cmd_icon[k] = v.get("render_icon", 
                                                "wysihtml5.widgets.render_blank")
                if v.get("render_dialog", False):
                    self.render_cmd_dialog[k] = v["render_dialog"]
            else: 
                self.render_cmd_icon[k] = "wysihtml5.widgets.render_blank"
                if v.get("render_dialog", False):
                    self.render_cmd_dialog[k] = "wysihtml5.widgets.render_blank"

        super(Wysihtml5TextareaWidget, self).__init__(attrs=attrs)

    def render(self, name, value, attrs=None):
        if value is None: value = ''
        final_attrs = self.build_attrs(attrs, name=name)
        textarea_widget = '<textarea%s>%s</textarea>' % (
            flatatt(final_attrs),
            conditional_escape(force_text(value)))
        wid = final_attrs.get('id', 'unknown')
        toolbar_widget = self.render_toolbar_widget(wid)
        pos = wid.find('__prefix__')
        if pos != -1:
            js_widget = self.render_js_delay_widget(wid, pos)
        else:
            js_widget = self.render_js_init_widget(wid) 
            return mark_safe('<div style="display:inline-block">' +
                         toolbar_widget + 
                         textarea_widget + 
                         '</div>' +
                         js_widget)

    def render_toolbar_widget(self, id):
        widget = '\
<div id="%(id)s-toolbar" class="wysihtml5-editor-toolbar">\
  <div class="commands">' % { "id": id }
        widget += get_function(self.render_cmd_icon['formatBlockHeader'])(id)
        widget += get_function(self.render_cmd_icon['formatBlockParagraph'])(id)
        widget += get_function(self.render_cmd_icon['bold'])(id)
        widget += get_function(self.render_cmd_icon['italic'])(id)
        widget += get_function(self.render_cmd_icon['underline'])(id)
        widget += get_function(self.render_cmd_icon['justifyLeft'])(id)
        widget += get_function(self.render_cmd_icon['justifyCenter'])(id)
        widget += get_function(self.render_cmd_icon['justifyRight'])(id)
        widget += get_function(self.render_cmd_icon['insertOrderedList'])(id)
        widget += get_function(self.render_cmd_icon['insertUnorderedList'])(id)
        widget += get_function(self.render_cmd_icon['insertImage'])(id)
        widget += get_function(self.render_cmd_icon['createLink'])(id)
        widget += get_function(self.render_cmd_icon['insertHTML'])(id)
        widget += get_function(self.render_cmd_icon['foreColor'])(id)
        widget += get_function(self.render_cmd_icon['changeView'])(id)
        widget += '\
  </div>\
  <div class="wysihtml5-dialogs">'
        widget += get_function(self.render_cmd_dialog['createLink'])(id)
        widget += get_function(self.render_cmd_dialog['insertImage'])(id)
        widget += '\
  </div>\
</div>'
        return widget

    def render_js_delay_widget(self, id, position):
        options = {"id": id}
        options.update(self.editor_settings)
        if not options.get('name', None) or options['name'] == 'null':
            options['name'] = '"%s"' % id[3:]
        if not options.get('toolbar', None) or options['toolbar'] == 'null':
            options['toolbar'] = '"%s-toolbar"' % id
        options['prefixid'] = id[0:position]
        widget = '''
<script>
  setTimeout(function() {
    var id = '%(id)s';
    var name = '%(name)s';
    var toolbar = %(toolbar)s;
    if(typeof(window._wysihtml5_inited) == 'undefined') {
      window._wysihtml5_inited = []
    }
    if(typeof(window._wysihtml5_inited[id]) == 'undefined') {
      window._wysihtml5_inited[id] = true;
    } else {
      var totforms = parseInt(document.getElementById('%(prefixid)sTOTAL_FORMS').value)-1;
      var newid = id.replace(/__prefix__/, totforms);
      var name = name.replace(/__prefix__/, totforms);
      var newtoolbar = toolbar.replace(/__prefix__/, totforms);
      django.jQuery('#'+toolbar).attr('id', newtoolbar);
      console.log(newid);
      if(document.getElementById(newid)) {
        new wysihtml5.Editor(newid,{
          name:                 name,
          style:                %(style)s,
          toolbar:              newtoolbar,
          autoLink:             %(autoLink)s,
          parserRules:          %(parserRules)s,
          parser:               %(parser)s,
          composerClassName:    %(composerClassName)s,
          bodyClassName:        %(bodyClassName)s,
          useLineBreaks:        %(useLineBreaks)s,
          stylesheets:          %(stylesheets)s,
          placeholderText:      %(placeholderText)s,
          allowObjectResizing:  %(allowObjectResizing)s,
          supportTouchDevices:  %(supportTouchDevices)s
        });
      }
    }
  }, 0);
</script>''' % options
        return widget

    def render_js_init_widget(self, id):
        options = {"id": id}
        options.update(self.editor_settings)
        if options.get('toolbar', 'null') == 'null':
            options['toolbar'] = '"%s-toolbar"' % id
        widget = '''
<script>
  new wysihtml5.Editor("%(id)s",{
    name:                 %(name)s,
    style:                %(style)s,
    toolbar:              %(toolbar)s,
    autoLink:             %(autoLink)s,
    parserRules:          %(parserRules)s,
    parser:               %(parser)s,
    composerClassName:    %(composerClassName)s,
    bodyClassName:        %(bodyClassName)s,
    useLineBreaks:        %(useLineBreaks)s,
    stylesheets:          %(stylesheets)s,
    placeholderText:      %(placeholderText)s,
    allowObjectResizing:  %(allowObjectResizing)s,
    supportTouchDevices:  %(supportTouchDevices)s
  });
</script>''' % options
        return widget
